//****************************************************
//*****  128x64 SPI COLOR LCD  Graphic routines  *****
//*************************** By Minarai Shokunin ****
//*************************** 2014/04/13          ****
//****************************************************

#include "p24FJ64GA002.h"
#include "font8x8.h"

#define uint unsigned int
#define uchar unsigned char

extern const unsigned char font8x8[];

extern void PutPixel(uint,uint,uint);
extern void dsp_image(int,int,int,int,char *);
extern void dsp_image16(int,int,int,int,int *);
extern void dsp_single_colour(uchar,uchar);

// ************ prototypes ***************************
void GE_SCALE(int);
void GR_FILL(unsigned int);
void GR_PSET(int,int,unsigned int);
void GR_PUTCH(int,int,unsigned int,unsigned int,unsigned int);
void GR_LINE(int,int,int,int,unsigned int);
void GR_BOXFILL(int,int,int,int,unsigned int);
void GR_BOX(int,int,int,int,unsigned int);
void GR_PUTSTR(int,int, char *, unsigned int,unsigned int);
void GR_PUTIMAGE(int,int,int,int,char *);


// ************ Values ***************************
int nGR_SCALE = 1;
int char_work[32*32];

// ************ Initialization ***************************

//********** Scale Set *************
void GR_PUTIMAGE(int x, int y, int w, int h,char *data) {
	dsp_image(y,x,w,h,data);
}


//********** Scale Set *************
void GR_SCALE(int scale) {
	nGR_SCALE = scale;
}

//********** Fill data *************
void GR_FILL(unsigned int cl) {
	unsigned char dh,dl;
	dh = (cl >> 8);
	dl = cl & 0xff;
    dsp_single_colour(dh,dl);
}

//********** box *************
void GR_BOX(int x0,int y0,int w, int h,unsigned int cl) {
	GR_LINE(x0,y0,x0+w,y0,cl);
	GR_LINE(x0,y0,x0,y0+h,cl);
	GR_LINE(x0+w,y0,x0+w,y0+h,cl);
	GR_LINE(x0,y0+h,x0+w,y0+h,cl);
}
//********** boxfill *************
void GR_BOXFILL(int x0,int y0,int w, int h,unsigned int cl) {
	int i;
	for (i=0;i<h;i++) {
		GR_LINE(x0,y0+i,x0+w-1,y0+i,cl);
	}
}

//********** Put string *************
void GR_PUTSTR(int x,int y, char *dt, unsigned int col,unsigned bg) {
    char *p0;
	int x0;	
	
	p0 = dt;
	x0 = x;

	while (*p0 != 0x00) {
		GR_PUTCH(x0,y,*p0++,col,bg);
		x0 += 8*nGR_SCALE;;
	}	
}

//********** Put Number *************
void GR_PUTNUM(int x,int y,int v, unsigned int col,unsigned int bg) {

	int i,x0,vv;
	
	x0 = x+3*8*nGR_SCALE;
	vv=v;

	for (i=0;i<4;i++) {
		if ((i==0)||(vv>=0)) {
			GR_PUTCH(x0,y,0x30+(vv%10),col,bg);
		}
		x0 -= nGR_SCALE*8;
		vv /=10;
	}
}



//********** Put character *************

void GR_PUTCH(int x,int y,unsigned int dt, unsigned int col,unsigned int bg) {

	int p0,i,j,k,l;
	unsigned int c0;
	unsigned int cl;	

	p0 = (dt - 0x20) * 8;

	for (i=0;i<8;i++) {
		c0 = font8x8[p0+i];
		for (j=0;j<8;j++) {
			if ((c0 & 0x01) != 0) {
				cl = col;
			}else{
				cl = bg;				
			}
			for (k=0;k<nGR_SCALE;k++) {
				for (l=0;l<nGR_SCALE;l++) {
					char_work[(j*nGR_SCALE+l)+(i*nGR_SCALE+k)*(8*nGR_SCALE)]= cl;
				}
			}	
			c0 >>= 1;
		}
	}
	dsp_image16(y,x,8*nGR_SCALE,8*nGR_SCALE,char_work);
}



//********** Pset *************
void GR_PSET(int x,int y,unsigned int col) {

	if ((x<0)||(x>159)||(y<0)||(y>127)) {return ;}	 
	PutPixel(y+2,160-x,col);

}

//********** Line *************

void GR_LINE(int x0,int y0,int x1,int y1,unsigned int col)
{
  int i,dx,dy,e,sx,sy,xx0,yy0;

  xx0=x0;
  yy0=y0;

  if (x1 > x0) {
		dx = x1-x0;
		sx = 1;
  }else{
		dx = x0-x1;
		sx = -1;
  }

  if (y1 > y0) {
		dy = y1-y0;
		sy = 1;
  }else{
		dy = y0-y1;
		sy = -1;
  }

  if ( dx > dy ) {
    e = -dx;
    for ( i = 0 ; i <= dx ; i++ ) {
      GR_PSET(xx0,yy0, col );
      xx0 += sx;
      e += 2 * dy;
      if ( e >= 0 ) {
        yy0 += sy;
        e -= 2 * dx;
      }
    }
  } else {
    e = -dy;
    for ( i = 0 ; i <= dy ; i++ ) {
      GR_PSET(xx0,yy0, col );
      yy0 += sy;
      e += 2 * dx;
      if ( e >= 0 ) {
        xx0 += sx;
        e -= 2 * dy;
      }
    }
  }
}
