//****************************************************
//***** Pocket MIKU LCD          *********************
//*************************** By Minarai Shokunin ****
//*************************** 2014/04/26          ****
//****************************************************

#include "p24FJ64GA002.h"
#include "miku.h"
#define uint unsigned int
#define uchar unsigned char

// ************ CONFIGRATION BITS ********************
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF )
_CONFIG2(IESO_OFF & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE)

// ************ prototypes ***************************
extern void GR_SCALE(int);
extern void GR_FILL(unsigned int);
extern void GR_PSET(int,int,unsigned int);
extern void GR_PUTCH(int,int,unsigned int,unsigned int,unsigned int);
extern void GR_LINE(int,int,int,int,unsigned int);
extern void GR_BOXFILL(int,int,int,int,unsigned int);
extern void GR_BOX(int,int,int,int,unsigned int);
extern void GR_PUTSTR(int,int,char *, unsigned int,unsigned int);
extern void GR_PUTNUM(int,int,int, unsigned int,unsigned int); 
extern void GR_PUTIMAGE(int,int,int,int,char *);

extern void LCD_CtrlWrite_IC(uchar);
extern void LCD_DataWrite_IC(uchar);  
extern void LCD_DataWrite(uchar,uchar);
extern void write_command(uchar);
extern void write_data(uchar);
extern void Reset();
extern void lcd_initial();
extern void RamAdressSet();
extern void PutPixel(uint,uint,uint);
extern void RedLine();
extern void dsp_single_colour(uchar,uchar);
extern void Output_Pixel(uint,uint);
extern void delay(uint);

unsigned int get_adc(int);

int hello[] = {4,2,1,1,0,0,1,0,4,2,0xff};
int chocho[] = {1,4,4,1,4,4,0,4,0,1,4,0,3,0,4,0,1,0,1,0,0,0,2,0,1,4,0,1,0xff};
int sakura[] = {0,2,0,0,2,0,0,4,1,4,4,0,0,0,1,0,0,2,0,1,1,0xff};
int hotaru[] = {4,0,2,4,1,0,0,1,0,4,4,2,2,1,2,1,4,2,2,1,1,0,0,3,2,2,0xff};
int iroha[] = {1,4,0,1,4,3,4,1,1,2,2,4,0,0,4,0,1,4,2,3,0,0,2,0xff};
int doremi[]={0,0,4,4,0,0,1,4,4,3,3,1,0,0,4,4,0,0,1,4,4,1,1,1,0,0,4,0xff};

// *********************************************
#define KEY_OFF		0
#define KEY_PUSHING	1
#define KEY_PUSHED	2
#define KEYS		9
#define VOL_MAX 	8
#define	VOL_MIN		0

//extern char miku[];

int key_state[KEYS];
int key_vup = KEY_OFF;
int key_vupcount = 0;
int key_vdown = KEY_OFF;
int key_vdowncount = 0;
int volume =5;
int	key_port[] ={0, 1,0,1 ,1 ,1  ,0,0 , 1};
int key_bits[] ={1,16,4,32,64,128,8,16,1024};		//*** 1 = PORTB bits, 0 = PORTA bits ********

// *********************************************
// ************ MAIN ***************************
// *********************************************
int kb_statea;
int kb_stateb;
int board_value;
int board_onoff;
int pnt;
int sw_state_A,sw_state_B;
int sw_state_A_pre,sw_state_B_pre;

int main(void)
{
	int i;
	int x,x0;
	int ch_flag = 0;
	int mode = 0;
	int lip = 0;

	pnt =0;x=0;x0=0;

	_RCDIV = 0;							// PostScaler 32MHz
	TRISB = 0b0000011111111111;			// PORTB 15-11:output , 10-0 input
	TRISA = 0b1111111111111111;			// PORTA all input

	// *********** ADC initialize *******************
	CMCON 				= 0;
	AD1PCFG				= 0xFFFD;		// Use AN1 as analog
	AD1CON1 			= 0x2008;
	AD1CON2 			= 0x0000;
	AD1CON3 			= 0x0000;
	AD1CHS  			= 0x0000;
	AD1CSSL 			= 0x0000;

//	 *********** valiables Itinialize ***************
	for (i=0;i<KEYS;i++) {
		key_state[i] = KEY_OFF;
	}
//	 *********** Interval timer Itinialize ***************
    T1CONbits.TCKPS 	= 1;   // Timer1 Pre scaler 1:8
	PR1 				= 30; // 10us interval timer
    T1CONbits.TON   	= 1;    // Timer1 ON
 	IEC0bits.T1IE = 1;  // Timer1 interrupt enable

    delay(500);	
	Reset();
    delay(500);	
	lcd_initial();
     delay(500);	
 	write_command(0x2C);
    delay(500);	
    GR_FILL(0x0000);
    delay(500);	
	GR_SCALE(2);
	GR_PUTSTR(0,0,  "**********",0x0055,0x0000);
	GR_PUTSTR(0,16, "* POCKET *",0x0055,0x0000);
	GR_PUTSTR(0,32, "*  MIKU  *",0x0055,0x0000);
	GR_PUTSTR(0,48, "**********",0x0055,0x0000);
    delay(15000);	
	GR_PUTSTR(5,60,"KIDOU",0xfff0,0x0000);
	GR_PUTSTR(5,80,"SIMASITA",0xfff0,0x0000);
    delay(5000);
    GR_FILL(0x0000);
	for (i=0;i<8;i++) {
		GR_BOX(0+i,0+i,160-i*2,128-i*2,i*2);
	}	
	GR_PUTIMAGE(10,10,100,100,(char*)miku);
	GR_SCALE(1);
	GR_PUTSTR(120,20,"NORM",0xffff,0x0000);
	GR_PUTSTR(120,40,"VOL ",0xffff,0x0000);
	GR_PUTNUM(120,60,volume,0xffff,0x0000);
	GR_PUTSTR(120,80,"VON ",0xffff,0x0000);

	sw_state_A_pre = sw_state_A;
	sw_state_B_pre = sw_state_B;

	while(1) {
		
		//*********** No touching ***********

		if (board_onoff > 900) {
			GR_PUTIMAGE(50,62,30,30,(char *)miku1);
			if (ch_flag ==0) {
				pnt++;;
				ch_flag =1;
			}

		//********** If touching *************

		}else{

			//********* Check stylus position ********************

			ch_flag = 0;
			x = (board_value*40 - 158*40) / 1000;
			if (x>26) {x=26;}				
			GR_PUTNUM(120,100,board_value,0xffff,0x0000);
			if (x0 != x) {
				GR_PUTCH(10+x0*4,110,' ',0xff00,0x0000);
				GR_PUTCH(10+x*4,110,'*',0xff00,0x0000);
				x0 = x;
			}

			//******** Select lip pattern ********************

			switch (mode) {
				case 0: lip = doremi[x];break;
				case 1: lip = 0;break;
				case 2: lip = 1;break;
				case 3: lip = 2;break;
				case 4: lip = 3;break;
				case 5: lip = 4;break;
				case 6: if (hello[pnt] == 0xff) pnt =0;
						lip = hello[pnt];
						break;
				case 7: if (chocho[pnt] == 0xff) pnt =0;
						lip = chocho[pnt];
						break;
				case 8: if (sakura[pnt] == 0xff) pnt =0;
						lip = sakura[pnt];
						break;
				case 9: if (hotaru[pnt] == 0xff) pnt =0;
						lip = hotaru[pnt];
						break;
				case 10: if (iroha[pnt] == 0xff) pnt =0;
						lip = iroha[pnt];
						break;
			}

			//*********** Disp Lip *******************

			switch (lip) {
				case	0: GR_PUTIMAGE(50,62,30,30,(char *)miku2);break;
				case	1: GR_PUTIMAGE(50,62,30,30,(char *)miku3);break;
				case	2: GR_PUTIMAGE(50,62,30,30,(char *)miku4);break;
				case	3: GR_PUTIMAGE(50,62,30,30,(char *)miku5);break;
				case	4: GR_PUTIMAGE(50,62,30,30,(char *)miku6);break;
			}

		}

		//*********** Display sw status if changed *******************

		if ((sw_state_A_pre != sw_state_A)||(sw_state_B_pre != sw_state_B)) {

			sw_state_A_pre = sw_state_A;
			sw_state_B_pre = sw_state_B;

			if ((key_state[0]==1)&&(key_state[1]==1)) {
				GR_PUTSTR(120,20,"NORM",0xffff,0x0000);
				mode = 0;
			}else{
				if (key_state[1]==0) {
					if (key_state[2] == 1) { GR_PUTSTR(120,20,"'O' ",0xffff,0x0000);mode = 5;pnt = 0;};
					if (key_state[3] == 1) { GR_PUTSTR(120,20,"'E' ",0x0ff0,0x0000);mode = 4;pnt = 0;};
					if (key_state[4] == 1) { GR_PUTSTR(120,20,"'U' ",0x00ff,0x0000);mode = 3;pnt = 0;};
					if (key_state[5] == 1) { GR_PUTSTR(120,20,"'I' ",0x0ff0,0x0000);mode = 2;pnt = 0;};
					if (key_state[6] == 1) { GR_PUTSTR(120,20,"'A' ",0xff00,0x0000);mode = 1;pnt = 0;};
				}else{
					if (key_state[2] == 1) { GR_PUTSTR(120,20,"P5 I",0xff00,0x0000);mode = 10;pnt = 0;};
					if (key_state[3] == 1) { GR_PUTSTR(120,20,"P4 H",0x0ff0,0x0000);mode = 9;pnt = 0;};
					if (key_state[4] == 1) { GR_PUTSTR(120,20,"P3 S",0x00ff,0x0000);mode = 8;pnt = 0;};
					if (key_state[5] == 1) { GR_PUTSTR(120,20,"P2 C",0x0ff0,0x0000);mode = 7;pnt = 0;};
					if (key_state[6] == 1) { GR_PUTSTR(120,20,"P1 K",0xff00,0x0000);mode = 6;pnt = 0;};
				}
			}

			//*********** Disp volume *******************

			if (key_vup == KEY_PUSHED) {
				if (volume < VOL_MAX) {
					volume++;
				}
				key_vup = KEY_OFF;
			}
			if (key_vdown == KEY_PUSHED) {
				if (volume > VOL_MIN) {
					volume--;
				}
				key_vdown = KEY_OFF;
			}
			GR_PUTSTR(120,40,"VOL ",0xffff,0x0000);
			GR_PUTNUM(120,60,volume,0xffff,0x0000);

			//*********** Disp vibrato status *******************

			if (key_state[0]==1) {
				GR_PUTSTR(120,80,"VON ",0xffff,0x0000);
			}else{
				GR_PUTSTR(120,80,"VOFF",0xffff,0x0000);
			}
		}
	}
}



// *********************************************
// ************ Timer Interrupt ****************
// *********************************************

void __attribute__((interrupt, auto_psv)) _T1Interrupt(void) {

	static int state =0;
	static int adcn[20];
	static int cnt=0;
	static int pb_his[5];
	static int pa_his[5];
	int i;
	int err;
	int sum;
	int key_same;

	switch (state)  {
		case 0:
			//******* Get scan timing ***************

			if ((PORTBbits.RB10 ==1)||(PORTBbits.RB7 ==1)) {
					state = 1;
			}
			break;
		case 1:
			board_onoff=get_adc(1);

			//******* Chattering rejection ***************
			key_same = 1;
			pb_his[4] = PORTB & 0x0fff;
			pa_his[4] = PORTA & 0x0fff;

			for (i=0;i<4;i++) {
				if ((pa_his[i] != pa_his[4])||(pb_his[i] != pb_his[4])) {
					key_same = 0;
				}
				pa_his[i] = pa_his[i+1];
				pb_his[i] = pb_his[i+1];
			}

			//******* Get switch status if stable ***************

			if (key_same == 1) {

				sw_state_A = pa_his[0];
				sw_state_B = pb_his[0];

				for (i=0;i<KEYS;i++) {
					if (key_port[i]==0) {
						if ((pa_his[0] & key_bits[i])==0) {
							key_state[i] = KEY_PUSHING;
						}else{
							key_state[i] = KEY_OFF;
						}
					}else{
						if ((pb_his[0] & key_bits[i])==0) {
							key_state[i] = KEY_PUSHING;
						}else{
							key_state[i] = KEY_OFF;
						}
					}
				}
			}

			//******* Set switch flags ***************

			if (key_state[7] == KEY_PUSHING) {
				if ((key_vdown==KEY_OFF)&&(key_vdowncount <10)) {
					key_vdowncount++;
				}
			}else{
				if ((key_vdowncount >5)) {
					key_vdowncount =0;
					key_vdown = KEY_PUSHED;
				}
			}
			if ((key_state[8] == KEY_PUSHING)) {
				if ((key_vup==KEY_OFF)&&(key_vupcount<10)) {
					key_vupcount++;
				}
			}else{
				if (key_vupcount >5) {
					key_vupcount =0;
					key_vup = KEY_PUSHED;
				}
			}
			state =2;
			cnt =0;
			break;	
		case 2:
			if ((PORTBbits.RB10 ==0)&&(PORTBbits.RB7 ==0)) {
				state = 3;
			}
			break;
		case 3:
			cnt++;
			if (cnt>=2) {
				sum = 0;err = 0;
				for (i=0;i<15;i++) {
					if ( ((adcn[i]-30)>adcn[i+1])||( (adcn[i]+30)<adcn[i+1]) ) {
						 err = 1;
					}  
					adcn[i]=adcn[i+1];sum += adcn[i];
				}
				adcn[15] = get_adc(1);
				if (err == 0) {
					board_value = sum/15;
				}
				state =0;
			}
			break;	
	
		default:
			state =0;
			break;	
	};
			
	// *** 荞ݏI@*********	
    IFS0bits.T1IF = 0; 		// Timer 1 Interrupt flag clear
}

unsigned int get_adc(int ch) {
	int ret,t;

	AD1CHS = ch;
	AD1CON1bits.ADON = 1;		// ADC Start
	AD1CON1bits.SAMP = 1;		// Sampling start
	for (t=0;t<10;t++) {};
	AD1CON1bits.SAMP = 0;
	while(!AD1CON1bits.DONE) {};
	ret = ADC1BUF0;
	return ret;
}

